import { Controller } from "@hotwired/stimulus";

function debounce(func, wait) {
  let timeout;
  return function (...args) {
    clearTimeout(timeout);
    timeout = setTimeout(() => func.apply(this, args), wait);
  };
}

export default class extends Controller {
  static targets = ["input", "clear", "list"];

  connect() {
    this.filter = debounce(this.filter_nodebounce.bind(this), 300);
  }

  expand() {
    if(this.listTarget.children.length == 0) {
      const conference = this.element.dataset.conference;
      const session = this.element.dataset.session;
      const url = `/conferences/${conference}/sessions/${session}/assignments`;

      const template = document.getElementById("template_filteredlist_options").content;
      const clone = document.importNode(template, true);
      clone.querySelectorAll("a.add_button").forEach(el => {
        el.setAttribute("href", `${url}?user_id=${el.dataset.user}`);
        el.setAttribute("data-turbo-frame",
          this.element.closest('turbo-frame[data-controller="session"]').id);
      });
      this.listTarget.appendChild(clone);
    }

    this.listTarget.classList.remove("hidden");
    this.inputTarget.classList.remove("rounded-b-md");
    this.filter_nodebounce();
  }

  contract() {
    this.listTarget.classList.add("hidden");
    this.inputTarget.classList.add("rounded-b-md");
    this.filter_nodebounce();
  }

  filter_nodebounce() {
    const query = this.inputTarget.value.toLowerCase();

    if (query || !this.listTarget.classList.contains("hidden")) {
      this.clearTarget.classList.remove("hidden");
    } else {
      this.clearTarget.classList.add("hidden");
    }

    // Filter options by the `data-filteredlist-match` attribute
    for (const option of this.listTarget.children) {
      if (option.dataset.filteredlistMatch.includes(query)) {
        option.classList.remove("hidden");
      } else {
        option.classList.add("hidden");
      }
    }
  }

  clear_filter() {
    this.inputTarget.value = '';
    this.filter_nodebounce();
    this.contract();
  }
}
